/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.reporting.server;

import java.io.File;
import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.core.framework.Platform;
import org.eclipse.birt.report.engine.api.EngineConfig;
import org.eclipse.birt.report.engine.api.IReportEngine;
import org.eclipse.birt.report.engine.api.IReportEngineFactory;
import org.ow2.orchestra.util.Misc;
import org.ow2.util.substitution.engine.DefaultSubstitutionEngine;
import org.ow2.util.substitution.resolver.ChainedResolver;
import org.ow2.util.substitution.resolver.PropertiesResolver;
import org.ow2.util.substitution.resolver.RecursiveResolver;

import javax.servlet.ServletContext;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

// BirtEngine.java is used to initialize the Report Engine
public class BirtEngine {

  private static IReportEngine birtEngine = null;
  private static Properties configProps = new Properties();
  private final static String configFile = "BirtConfig.properties";
  private static final Logger LOG = Logger.getLogger(BirtEngine.class.getName());

  /**
   * The init() fonction
   */

  public static synchronized void initBirtConfig() {
    loadEngineProps();

    Misc.fastDynamicLog(LOG, Level.INFO, "Birt Engine initialization");

  }

  /**
   * getBirtEngine method
   *
   * @param sc
   * @return birtEngine
   */
  public static synchronized IReportEngine getBirtEngine(ServletContext sc) throws BirtException {
    if (birtEngine == null) {
      EngineConfig config = new EngineConfig();
      if (configProps != null) {
        String logLevel = configProps.getProperty("logLevel");

        Level level = Level.OFF;
        try {
          level = Level.parse(logLevel);
        } catch (IllegalArgumentException iae) {
          Misc.fastDynamicLog(LOG, Level.SEVERE, "Illegal value in Birt configuration file for log level. Birt logs disabled.");
        }

        // update the properties resolver with system properties
        PropertiesResolver sysPropResolver = new PropertiesResolver(System.getProperties());
        // update the properties resolver with Birt properties
        PropertiesResolver configPropResolver = new PropertiesResolver(configProps);

        // Chain both properties resolvers (first match is returned)
        ChainedResolver chainedResolver = new ChainedResolver();
        chainedResolver.getResolvers().add(configPropResolver);
        chainedResolver.getResolvers().add(sysPropResolver);

        // define the substitution engine
        DefaultSubstitutionEngine dse = new DefaultSubstitutionEngine();

        //Create a recursive resolver
        RecursiveResolver recursiveResolver = new RecursiveResolver(dse, chainedResolver);

        // define the properties of base of substitution engine
        dse.setResolver(recursiveResolver);

        String logDir = configProps.getProperty("logDirectory", "${java.io.tmpdir}" + File.separator + "birt" + File.separator + "log");
        logDir = dse.substitute(logDir);
        config.setLogConfig(logDir, level);

        String tmpDir = configProps.getProperty("tmpDir", "${java.io.tmpdir}" + File.separator + "birt");
        tmpDir = dse.substitute(tmpDir);
        config.setTempDir(tmpDir);

      }

      // This which will load the appropriate plug-ins
      Platform.startup(config);
      Misc.fastDynamicLog(LOG, Level.INFO,
              " Start up the OSGI framework for Birt Engine");
      IReportEngineFactory factory = (IReportEngineFactory) Platform
              .createFactoryObject(IReportEngineFactory.EXTENSION_REPORT_ENGINE_FACTORY);

      if (factory == null) {
        throw new BirtException(
                "factory is null, the appropriate plugins have not been load,  GENERATION_EXCEPTION_");
      }

      birtEngine = factory.createReportEngine(config);

    } // end if
    return birtEngine;
  }

  /**
   * Method to destroy the servlet
   */
  public static synchronized void destroyBirtEngine() {
    if (birtEngine == null) {
      return;
    }
    birtEngine.destroy();
    Platform.shutdown();
    birtEngine = null;
  }

  private static void loadEngineProps() {
    try {
      ClassLoader cl = Thread.currentThread().getContextClassLoader();
      InputStream in = null;
      in = cl.getResourceAsStream(configFile);

      // ///////////////////to check
      if (in == null)
        throw new IOException(
                "the configuration file BirtConf.properties is not found ");

      configProps.load(in);

      in.close();
    } catch (IOException e) {

      e.printStackTrace();

    }

  }
}
