/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.reporting.server;

import org.eclipse.birt.report.engine.api.script.IReportContext;
import org.eclipse.birt.report.engine.api.script.IScriptedDataSetMetaData;
import org.eclipse.birt.report.engine.api.script.IUpdatableDataSetRow;
import org.eclipse.birt.report.engine.api.script.ScriptException;
import org.eclipse.birt.report.engine.api.script.eventadapter.ScriptedDataSetEventAdapter;
import org.eclipse.birt.report.engine.api.script.instance.IDataSetInstance;
import org.ow2.orchestra.facade.AccessorUtil;
import org.ow2.orchestra.facade.QueryDefinitionAPI;
import org.ow2.orchestra.facade.def.ProcessDefinition;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.exception.OrchestraException;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

public class DataSetEventHandlerProcessStateFiltered extends
        ScriptedDataSetEventAdapter {

  /**
   * IReportContext is An interface used to share information between the
   * event methods in scripting . Gives access to report parameters and
   * configuration values. Also provides a way for the report developer to
   * register and retrieve custom properties.
   */

  final static String PROCESS_STATE_COLUMN_NAME = "PROCESS_STATE";
  private String retrievedParameterUrl = null;
  private String retrievedParameterObjectName = null;

  private Iterator<ProcessState> statesIterator;

  public DataSetEventHandlerProcessStateFiltered() {
  }

  /**
   * The open methode is called when the data set is opened , use this method
   * to initialize variables and to prepare for fetching rows Need to add
   * throws exception in each
   */

  @Override
  public void open(IDataSetInstance dataSet) throws ScriptException {
    QueryDefinitionAPI api;
    try {
      api = AccessorUtil.getQueryDefinitionAPI(retrievedParameterUrl, retrievedParameterObjectName);
    } catch (OrchestraException e) {
      throw new ScriptException("Unable to access to Orchestra API", e);
    }

    List<ProcessDefinition> processes = api.findProcessDefinitions();

    HashMap<String, ProcessState> statesByProcessesMap = new HashMap<String, ProcessState>();
    if (processes != null) {
      for (ProcessDefinition process : processes) {
        if (!statesByProcessesMap.containsKey(process.getName())) {
          statesByProcessesMap.put(process.getName(), process.getState());
        } else if (!ProcessState.UNDEPLOYED.equals(process.getState())) {
          ProcessState currentState = statesByProcessesMap.get(process.getName());
          if (!ProcessState.ACTIVE.equals(currentState)
                  && (ProcessState.ACTIVE.equals(process.getState()) ||
                  (ProcessState.RETIRED.equals(process.getState()) && ProcessState.UNDEPLOYED.equals(currentState)))) {
            statesByProcessesMap.put(process.getName(), process.getState());
          }
        }
      }
    }
    statesIterator = statesByProcessesMap.values().iterator();
  }

  @Override
  public boolean fetch(IDataSetInstance dataSet, IUpdatableDataSetRow row) throws ScriptException {
    if (!statesIterator.hasNext()) {
      return false;
    }
    ProcessState value = statesIterator.next();
    if (ProcessState.ACTIVE.equals(value)) {
      row.setColumnValue(PROCESS_STATE_COLUMN_NAME, "Active");
    } else if (ProcessState.RETIRED.equals(value)) {
      row.setColumnValue(PROCESS_STATE_COLUMN_NAME, "Retired");
    } else {
      row.setColumnValue(PROCESS_STATE_COLUMN_NAME, "Undeployed");
    }
    return true;
  }

  @Override
  public boolean describe(IDataSetInstance dataSet,
                          IScriptedDataSetMetaData metaData) throws ScriptException {
    super.describe(dataSet, metaData);
    metaData.addColumn(PROCESS_STATE_COLUMN_NAME, String.class);
    return true;
  }

  @Override
  public void beforeOpen(IDataSetInstance dataSet,
                         IReportContext reportContext) throws ScriptException {
    /**
     * IReportContext is the report context used to evaluate the java-based
     * script.
     */
    super.beforeOpen(dataSet, reportContext);
    retrievedParameterUrl = (String) reportContext.getParameterValue("PARAMBIRT_ORCHESTRA_URL");
    retrievedParameterObjectName = (String) reportContext.getParameterValue("PARAMBIRT_ORCHESTRA_OBJECTNAME");
  }

  @Override
  public void close(IDataSetInstance dataSet) throws ScriptException {
    super.close(dataSet);
  }
}
